#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <QDebug>

//////////////////////////
// Cabeceras QVision
#include <qvgui/qvcamerawidget.h>
#include <qvgui/qvworkeruserinterface.h>
#include <qvmplayercamera/qvmplayercamera.h>
#include <qvcore/qvapplication.h>
#include <qvipp/qvipp.h>

#include "siftextractor.h"

////////////////////////
// Class MyWorker
class MyWorker: public QVWorker
	{
	Q_OBJECT

	private:
		QVMPlayerCamera *cam;

	public:
		MyWorker(QString name): QVWorker(name)
			{
			add(cam = new QVMPlayerCamera());
			addProperty("Threshold",	5.0,		"Threshold");
			addProperty("Scharr",		true,		"Use Scharr");
			addProperty("Points",		10,		"# of points to be displayed");
			addProperty("Video",		QString(),	"Url to video");

			setPropertyRange("Threshold", 0.0, 10.0);
			setPropertyRange("Points", 1, 100);
			}

		void init()
			{
			QString videoFile = getPropertyValue<QString>("Video");

			if (videoFile == QString())
				{ status = QVWorker::Finished; }
			else
				{
				if(!cam->isClosed())
					cam->closeCam();

				if(cam->openCam(videoFile, QVMPlayerCamera::RealTime | QVMPlayerCamera::RGBMencoderMode) == -1)
					qFatal("openCam devuelve -1 en cámara. ¿bad URL?\n");
				else
					status = QVWorker::Running;
				}
			}

		void worker()
			{
			qDebug() << "MyWorker::worker()";

			bool scharr =		getPropertyValue<bool>("Scharr");
			int sizeMax =		getPropertyValue<int>("Points");
			double threshold =	getPropertyValue<double>("Threshold");

			QVImage<uChar,3> image;
			cam->grab(image);

			if (cam->isClosed())
				{
				qDebug() << "void worker(): camera state closed. Finishing...";
				finish();
				qDebug() << "void worker() <- return";
				return;
				}

			uInt	rows = cam->getRows(),
				cols = cam->getCols();

			timeFlag("Image grab");

			/////////////////
			// SIFT
			QList<QVSIFTDescriptor>	descriptors = getSIFTDescriptors(image);
			timeFlag("Get SIFT descriptors");

			QVImage<uChar,3> descriptorsImg = image;
			drawSIFTDescriptors(descriptors, descriptorsImg);
			timeFlag("Draw SIFT descriptors");

			emit newFrame("Input image", image);
			emit newFrame("Descriptors image", descriptorsImg);
			//qDebug() << "testSIFT " << testSIFT();
			qDebug() << "MyWorker::worker(): <- return";
			}
	};
