/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvcpuplot.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef CPUPLOT_H
#define CPUPLOT_H

#include <QApplication>
#include <QTimer>
#include <QTableView>

#include <qpainter.h>
#include <qwt_plot.h>
#include <qwt_plot_curve.h>
#include <qvutils/cpustat.h>

#define MAX_HISTORY 60 // seconds

/// \cond Not documented By now
class QVCpuPlot : public QwtPlot
	{
	Q_OBJECT
	public:
		QVCpuPlot(CpuStat *, bool decorations = true, QWidget * = 0);

	public slots:
		void update();

	private:
		bool decorations;
		QwtPlotCurve **curve;
		double **data;
		double timeData[MAX_HISTORY];
		int dataCount, nFlags;
		CpuStat *cpuStat;
		QTimer timer;

		void initPlot();
		void timerEvent(QTimerEvent *e);
		const double curveValue(int curve) const
			{
			if (curve == 0)
				return data[curve][0];
			else
				return data[curve][0] - data[curve-1][0];
			}

		friend class CpuPieMarker;
		class CpuPieMarker: public QwtPlotItem
			{
			public:
				CpuPieMarker(unsigned int nFlags, QwtPlotCurve **curves);
				virtual int rtti() const;
				virtual void draw(QPainter *p, const QwtScaleMap &, const QwtScaleMap &, const QRect &rect) const;
		
			private:
				unsigned int nCpuFlags;
				QwtPlotCurve **cpuCurve;
			};
	};

class QVCpuTableView: public QTableView
	{
	Q_OBJECT
	public:
		QVCpuTableView(CpuStat *cpuStat): QTableView()
			{
			this->cpuStat = cpuStat;
			}

		void update()
			{
			if (!cpuStat->isActive())
				return;

			QStringList timeFlags = cpuStat->getFlagNames();
			
			}

	private:
		CpuStat *cpuStat;
	};

/// \endcond Not documented By now
#endif
