/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvimagecanvas.h
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#ifndef QVIMAGECANVAS_H
#define QVIMAGECANVAS_H

#include <qvgui/qvcanvas.h>
#include <qvcore/qvpropertyholder.h>
#include <qvcore/qvworker.h>
#include <qvdta/qvpolyline.h>

/*!
@class QVImageCanvas qvimagecanvas.h qvgui/qvimagecanvas.h
@brief Viewer widget for @ref QVImage objects and other data types.

When created an object derived from this class, the interface opens a window that depicts the content of a property of type QVImage contained in a property holder (normally a @ref QVWorker object).

Usage is as follows: first create a QVImageCanvas object in the <i>main()</i> function of a program, then link a QVImage type property from a property holder:

\code
[...]

CannyWorker cannyWorker("Canny");

[...]

QVImageCanvas imageCanvas;
imageCanvas.linkProperty(cannyWorker,"Canny image");

[...]
\endcode

and latter call for the <i>exec()</i> function of the @ref QVApplication object of the program. This will automatically create a window like the following:

\image html qvimagecanvas_penguin.png

You can see it has a zoom number indicator (<i>z = 1</i>), horizontal and vertical rules (in pixels), and some buttons. These latter control zooming and moving around a zoomed area of the image mostly, and a detailed explanation of each one follows:

<table>
<tr><td><b>Zoom in button</b> <img src="qvimagecanvas_zoomin.png" /></td><td>This button zooms the image in. Canvas window doesn't change size, but the area of the image displayed becomes smaller by a factor of 4 (width and height gets divided by 2). The label of the left botton corner displays the text
\code
z=<zoom>
\endcode
where <i>&lt;zoom&gt;</i> is a number indicating the zoom factor that divides width and height in the actual zoom level.</td></tr>

<tr><td><b>Zoom out button</b> <img src="qvimagecanvas_zoomout.png" /></td><td>This button divides zoom factor by 2, if it is equal or greater than 2. If the canvas window is bigger than the image at the final zoom, it is adjusted to the size of the latter.</td></tr>

<tr><td><b>Zoom restore button</b> <img src="qvimagecanvas_zoomrestore.png" /></td><td>This button sets zoom factor by 1, and resizes canvas window to its original size.</td></tr>

<tr><td><b>Select zoom region button</b> <img src="qvimagecanvas_zoomregion.png" /></td><td>This button lets you select a rectangle in the image to zoom, adjusting zoom factor and canvas window size to it.</td></tr>

<tr><td><b>Drag mode</b> <img src="qvimagecanvas_drag.png" /></td><td>This button activates drag mode. With it you can move around the image displayed in the canvas window, if the zoom factor forces the canvas window to show only a subregion of the image, by holding click and dragging over the depicted image.</td></tr>
</table>

In the following figure you can see a zoomed canvas window depicting a subregion of the original image:

\image html qvimagecanvas_penguinpixels.png

Another interesting feature of image canvas is that at a zoom factor bigger or equal to 32, the canvas renders the gray-scale pixel value over every pixel if the image is gray-scale, or the three different values for each of the RGB channels over every pixel, if the image is RGB, as depicted below:

\image html qvimagecanvas_penguinpixelnumbers.png

\todo Advanced painting: pintar otros objetos, aparte de imagenes. Contornos, listas de puntos, etc...

@ingroup qvgui
*/
class QVImageCanvas: public QVCanvas, public QVPropertyHolder
	{
		Q_OBJECT
	public:
		QVImageCanvas(const QString name = QString (), QWidget *parent=0);
		void linkProperty(QVWorker &worker, const QString propertyName);
		void setLowHigh(float low,float high);
		/// \cond
		void viewer();	
		/// \endcond
	private:
		float _low,_high;
		void draw(const QList<qvdta::QVPolyline> &contours);
	};


/// \endcond Not documented By now
#endif
