/*
 *	Copyright (C) 2007. PARP Research Group.
 *	<http://perception.inf.um.es>
 *	University of Murcia, Spain.
 *
 *	This file is part of the QVision library.
 *
 *	QVision is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU Lesser General Public License as
 *	published by the Free Software Foundation, version 3 of the License.
 *
 *	QVision is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU Lesser General Public License for more details.
 *
 *	You should have received a copy of the GNU Lesser General Public
 *	License along with QVision. If not, see <http://www.gnu.org/licenses/>.
 */

/// @file qvparamsinspectorwidget.cpp
/// @brief File from the QVision library.
/// @author PARP Research Group. University of Murcia, Spain.

#include <qvgui/qvparamsinspectorwidget.h>

#include <QtGui/QWidget>

#include <qvcore/qvpropertyholder.h>
#include <qvcore/qvworker.h>
#include <qvgui/qvparamwidget.h>


QVParamsInspectorWidget::QVParamsInspectorWidget(QVPropertyHolder *holder, QWidget *parent):QWidget(parent)
	{
	//setGeometry(QRect(10, 20, 200, 0));

	vboxLayout = new QVBoxLayout(this);
	vboxLayout->setSpacing(0);

	QVWorker *worker;
	if((worker = dynamic_cast<QVWorker*>(holder)) != NULL)
		{
		QListIterator<QString> iter(worker->getTriggerList());
		while (iter.hasNext())
			{
			QString property(iter.next());
			QVWorkerTriggerWidget *triggerwidget = 
				new QVWorkerTriggerWidget(worker, property, this);
			vboxLayout->addWidget(triggerwidget);
			}
		}

	QListIterator<QString> iter(holder->getPropertyList());
	while (iter.hasNext())
		{
		QString property(iter.next());
		// Only input properties that are not linked will be displayed:
		if(not holder->isLinkedInput(property) and holder->isInput(property))
			{
			if(holder->isType<int>(property))
				{
				QVIntParamWidget *int_widget = new QVIntParamWidget(holder,this,property);
				vboxLayout->addWidget(int_widget);
				writeOutputProperties();
				connect(int_widget,SIGNAL(valueChanged(int)),this,SLOT(somePropertyChanged()));
				} /* if the property is int. */
			else if(holder->isType<double>(property))
				{
				QVDoubleParamWidget *double_widget = new QVDoubleParamWidget(holder,this,property);
				vboxLayout->addWidget(double_widget);
				writeOutputProperties();
				connect(double_widget,SIGNAL(valueChanged(double)),this,SLOT(somePropertyChanged()));
				} /* if the property is double. */
			else if(holder->isType<bool>(property))
				{
				QVBoolParamWidget *bool_widget = new QVBoolParamWidget(holder,this,property);
				vboxLayout->addWidget(bool_widget);
				writeOutputProperties();
				connect(bool_widget,SIGNAL(valueChanged(bool)),this,SLOT(somePropertyChanged()));
				} /* if the property is bool. */
			} /* if it is not linked and of the input type. */
		} /* while there are more properties. */

	vboxLayout->addStretch();

	}

void QVParamsInspectorWidget::somePropertyChanged()
	{
	writeOutputProperties();
	}
